/** ErrorList class
* @author: Filip Dvorak
* @version: 1.0
*/

package xelfi.compiler;

import java.util.*;

  /** This class keeps an information of all errors from compilation for one class. */
public class ErrorList
{
    /** class name for this error list. */
  protected String name;
  
  private ErrorDescription lastHighlighted;

    /** This is where the errors are kept. Elements are ErrorDescription classes. */
  protected Vector list;

    /** ErrorColorer for this class. */
  protected ErrorColorer colorer;

  ErrorList(String className)
  {
    list = new Vector(16, 16);
    name = className;
    colorer = ErrorColorerEvidence.getReference(className);
    lastHighlighted = null;
  }

  void addError(ErrorDescription desc)
  {
    list.addElement(desc);
    desc.setParent(this);
  }

    /** 
      Sets itself and ErrorColorer to highlight specified error. Typically called by ErrorDescription.
      @param err error to be highlighted. If null then error will be highligted.
     */
  public void setHighlightedError(ErrorDescription err)
  {
    colorer.setHighlight(lastHighlighted = err);
  }

    /** 
      @return an error last set to be highlighted.
     */
  public ErrorDescription getLastHighlightedError()
  {
    return lastHighlighted;
  }

    /** 
      Set's highlighted error to the next error from the current highlighted error. 
      @return next error which is being set to be highlighted. 
    */
  public ErrorDescription nextHighlightedError()
  {
    int next = (lastHighlighted == null ? 0 : list.indexOf(lastHighlighted) + 1);
    if(next < list.size())
    {
      ErrorDescription d = (ErrorDescription)list.elementAt(next);
      setHighlightedError(d);
      return d;
    }
    else return null;
  }

    /** 
      Set's highlighted error to the previous error from the current highlighted error. 
      @return previous error which is being set to be highlighted. 
    */
  public ErrorDescription previousHighlightedError()
  {
    int prev = (lastHighlighted == null ? 0 : list.indexOf(lastHighlighted) - 1);
    if(prev >= 0)
    {
      ErrorDescription d = (ErrorDescription)list.elementAt(prev);
      setHighlightedError(d);
      return d;
    }
    else return null;
  }

  String getClassName()
  {
    return name;
  }

    /** Called when removed from the ErrorDatabase (errors ere being cleared) */
  void removed()
  {
    colorer.setHighlight(null);
    ErrorColorerEvidence.closeReference(name);
  }
}


