/** Package list dialog
* @author: Filip Dvorak
* @version: 1.3
*/

package xelfi.compiler;

import java.awt.List;
import java.awt.*;
import java.util.*;

import xelfi.top.*;
import xelfi.top.visual.*;

class PackageListDialog extends Dialog
{
  protected CompilerOptions copt;
  Vector plist;
  List list;
  protected Button butOK, butCancel, butAdd, butRemove;

  PackageListDialog(CompilerOptions co)
  {
    super(TopLevel.getMenu(), "Edit package list", true);

    plist = co.getTmpList();
    copt = co;

    RowLayout dl = new RowLayout();
    setLayout(dl);
    RowLayoutProperties dp = new RowLayoutProperties(new Insets(2, 4, 2, 6), 0.88f, 0.02f, 0.05f);
    list = new List(plist.size(), false);
    add(list);
    dl.add(list, dp);
      // button panel
    Panel bp = new Panel();
    dp.compGrowth = 0.02f;
    dp.topSpaceGrowth = 0;
    dp.bottomSpaceGrowth = 0.03f;
    add(bp);
    dl.add(bp, dp);

      // button panel setup
    ColLayout bpl = new ColLayout();
    ColLayoutProperties bpp = new ColLayoutProperties(new Insets(2, 2, 2, 2), 0.03f);
    bp.setLayout(bpl);
      // add
    bpp.leftSpaceGrowth = 0.2f;
    bpp.rightSpaceGrowth = 0.04f;
    butAdd = new Button("Add");
    bp.add(butAdd);
    bpl.add(butAdd, bpp);
      // remove
    bpp.leftSpaceGrowth = 0;
    bpp.rightSpaceGrowth = 0.4f;
    butRemove = new Button("Remove");
    bp.add(butRemove);
    bpl.add(butRemove, bpp);
      // OK
    bpp.rightSpaceGrowth = 0.04f;
    butOK = new Button(" OK ");
    bp.add(butOK);
    bpl.add(butOK, bpp);
      // Cancel
    bpp.rightSpaceGrowth = 0.2f;
    butCancel = new Button("Cancel");
    bp.add(butCancel);
    bpl.add(butCancel, bpp);

    int size = plist.size();
    for(int i = 0; i < size; i++) list.addItem((String)plist.elementAt(i));

    pack();
    pack();
  }

  public boolean action(Event e, Object arg)
  {
    if(e.target == butAdd)
    {
      new AddListItemDialog(this).show();
      return true;
    }
    else if(e.target == butRemove)
    {
      onRemove();
      return true;
    }
    else if(e.target == butCancel)
    {
      hide();
      dispose();
      return true;
    }
    else if(e.target == butOK)
    {
      onOK();
      return true;
    }
    return super.action(e, arg);
  }

  public boolean handleEvent(Event evt)
  {
    switch(evt.id)
    {
      case Event.WINDOW_DESTROY:
        if(evt.target == this)
        {
          hide();
          dispose();
          return true;
        }
    }
    return super.handleEvent(evt);
  }

  public boolean keyDown(Event  e, int  key)
  {
    if(key == '\n')
    {
      onOK();
      return true;
    }
    else if(key == 'a' || key =='A')
    {
      handleEvent(new Event(butAdd, Event.ACTION_EVENT, null));
      return true;
    }
    else if(key == 'r' || key =='R')
    {
      handleEvent(new Event(butRemove, Event.ACTION_EVENT, null));
      return true;
    }
    else return false;
  }

  protected void onRemove()
  {
    int pos = list.getSelectedIndex();
    if(pos != -1)
    {
      list.delItem(pos);
      plist.removeElementAt(pos);
    }
  }

  protected void onOK()
  {
    copt.setTmpList(plist);
    hide();
    dispose();
  }

  public void show()
  {
    if(getPeer() == null) addNotify();
    Dimension scr = Toolkit.getDefaultToolkit().getScreenSize(), me = size();

    move((scr.width - me.width) / 2, (int)((scr.height - me.height) / 2.4));
    super.show();
  }

  public Dimension preferredSize()
  {
    Dimension pref = super.preferredSize();
    if(pref.width < 340) pref.width = 340;
    if(pref.height < 200) pref.height = 200;
    return pref;
  }

  public Insets insets()   // Under JDK 1.0.2 for win95 insets doesnt work. This patch should help.
  {
    Insets i = super.insets();
    if(getPeer() != null && (i.top < 0 || i.bottom < 0 || i.left < 0 || i.right < 0))
      return new Insets(28, 4, 4, 4);
    else return i;
  }
}







