/**
 * BreakpointDialog
 *
 * @author Roman Blazevic
 * @version 970208
 */

package xelfi.debugger;

import java.awt.*;
import xelfi.top.TopLevel;
import xelfi.top.TopDialog;

/**
 * This dialog is used to enter the class name and the line number
 * of the breakpoint.
 * Result from the dialog is send via parent.action() to the dialog's
 * parent.
 */

class BreakpointDialog extends TopDialog
{
	static final int ID_OK = 1;
	static final int ID_CANCEL = 0;

	private BreakpointsView parent;

	private Label label1;
	private Label label2;
	private TextField edit1;
	private TextField edit2;
	private Button okButton;
	private Button cancelButton;

	/**
	 * BreakpointDialog constructor.
	 */
	BreakpointDialog(BreakpointsView parent, String title)
	{
		super(TopLevel.getMenu(), false);
		setTitle(title);
		//super(title);

		this.parent = parent;

		setBackground(Color.lightGray);

		resize(insets().left + insets().right + 355, insets().top + insets().bottom + 235);
		setResizable(false);

		label1 = new Label("Class");
		add(label1);
		label1.reshape(insets().left + 15,insets().top + 36,90,16);

		label2 = new Label("Line");
		add(label2);
		label2.reshape(insets().left + 19,insets().top + 103,90,16);

		edit1 = new TextField(31);
		edit1.setBackground(Color.lightGray);
		add(edit1);
		edit1.reshape(insets().left + 18,insets().top + 60,315,32);

		edit2 = new TextField(31);
		edit2.setBackground(Color.lightGray);
		add(edit2);
		edit2.reshape(insets().left + 18,insets().top + 124,315,32);

		okButton = new Button("OK");
		add(okButton);
		okButton.reshape(insets().left + 87,insets().top + 177,70,28);

		cancelButton = new Button("Cancel");
		add(cancelButton);
		cancelButton.reshape(insets().left + 188,insets().top + 177,70,28);

		centerWindow();
	}

	/**
	 * Tahle metoda by mela byt implementovana jiz v TopFrame.
	 * FILIP
	 */
	void centerWindow()
	{
	}

	/**
	 * My action() implementation.
	 */
	public boolean action(Event event, Object what)
	{
		if ((event.target == okButton)
		    || (event.target == edit1)
			|| (event.target == edit2))
		{
			BreakpointDialogResult result;
			int lineNumber;

			try
			{
				lineNumber = Integer.valueOf(edit2.getText()).intValue();
			}
			catch (Exception e)
			{
				return false;
			}

			if (edit1.getText().compareTo("") == 0 || lineNumber < 1)
				return false;

			result = new BreakpointDialogResult();

			result.className = edit1.getText();
			result.lineNumber = lineNumber;
			result.returnCode = ID_OK;

			hide();
			parent.action(null, result);
			return true;
		}
		if (event.target == cancelButton)
		{
			BreakpointDialogResult result = new BreakpointDialogResult();

			result.returnCode = ID_CANCEL;
			parent.action(null, result);
			hide();
			return true;
		}
		return super.action(event, what);
	}

	/**
	 * My handleEvent() implementation.
	 */
	public boolean handleEvent(Event event)
	{
		if (event.id == Event.WINDOW_DESTROY)
		{
			BreakpointDialogResult result = new BreakpointDialogResult();

			result.returnCode = ID_CANCEL;
			hide();
			parent.action(null, result);
			return true;
		}
		return super.handleEvent(event);
	}

	/**
	 * My show() implementation.
	 */
	public void show()
	{
		super.show();
		edit1.requestFocus();
	}
}
