/**
 * Xelfi
 */

package xelfi.design.components.xawt;

/**
  *
  * @version 	2.34, 18 Mar 1997 (last change by IAN - added lines 67-70)
  * @author 	Petr Hamernik, Ian Formanek
  *
  */

import java.awt.*;
import java.io.*;
import xelfi.top.TopLevel;
import xelfi.design.forms.XelfiDesignForm;
import xelfi.design.util.*;
import xelfi.design.components.XelfiVisualComponent;
import xelfi.design.components.XelfiVisualContainer;
import xelfi.util.XelfiPath;
import xelfi.util.Debug;

public abstract class XAWTPanel extends Panel implements XelfiVisualComponent {
	// Common Properties
	protected Point position;
	protected Dimension size;
	protected String name = "";
	protected XelfiDesignForm parentForm = null;
	protected Component component = null;

	protected boolean autoSize = false;
	protected boolean bVisible = true;
	protected boolean parentColor = true;
	protected boolean parentFont = true;

	protected int sel = XFDesignConstants.SEL_UNSELECT;

	// U=Up, D=Down, L=Left, R=Right, M=Middle
	protected final static int DM_NONE	= 0;
	protected final static int DM_UL	= 1;
	protected final static int DM_ML	= 2;
	protected final static int DM_DL	= 3;
	protected final static int DM_UM	= 4;
	protected final static int DM_DM	= 6;
	protected final static int DM_UR	= 7;
	protected final static int DM_MR	= 8;
	protected final static int DM_DR	= 9;
	protected Point dragOrigin = null;
	protected int dragMode = DM_NONE;

	protected int borderSize = XFDesignOptions.borderSize;
	public final static boolean WITHOUT_DRAGBORDER = false;
	public final static boolean WITH_DRAGBORDER = true;

	public XAWTPanel(boolean bord) {
		setLayout(new BorderLayout());
		position = new Point(0,0);
		size = null;
		if (bord == WITH_DRAGBORDER)
			borderSize = XFDesignOptions.borderSize;
		else
			borderSize = 0;
	}

	public Insets insets() {
		// [IAN] in real mode we do not want to reserve space for drag border
		if (XFDesignOptions.designMode == XFDesignConstants.DM_REAL)
			return new Insets(0,0,0,0);
			
		if (sel == XFDesignConstants.SEL_UNSELECT)
			return new Insets(0,0,borderSize,0);
		else {
			int i = XFDesignOptions.selectionSize;
			return new Insets(i,i,i+borderSize,i);
		}
	}

	public void setComponent(Component comp) {
		add("Center", comp);
		component = comp;
	}

	public void setParentForm(XelfiDesignForm aForm) {
		parentForm = aForm;
	}

	public XelfiDesignForm getParentForm() {
		return parentForm;
	}

	// This method must be implemented by all components to ensure proper behaviour in layouts. 
	// With AWT components that extend XAWTPanel preferredSize must be redirected to actual AWT component
	// It will not be necessary with normal components since they have their own preferredSize methods
	// which work fine, unless (e.g. with Panel) they return (0,0) in some cases.
	public Dimension preferredSize() {
// [IAN]
		if (size == null)
			size = component.preferredSize();
//[/IAN]
		Dimension sz = component.preferredSize();
		if (XFDesignOptions.designMode == XFDesignConstants.DM_REAL)
			return sz;
		sz.height += borderSize;
		if (sel != XFDesignConstants.SEL_UNSELECT) {
			sz.height += 2*XFDesignOptions.selectionSize;
			sz.width += 2*XFDesignOptions.selectionSize;
		}
		return sz;
	}

	/** 
	 * if layout managers are used, then this method can return different size that the actual size
	 * actual is preferredSize of the component and the size property can have other stored size
	 * - this happens only when autoSize is OFF - it is meant that when user switches to layout manager
	 * and back, then it does not affect component sizes
	 */

	public Dimension getDesignSize() { 
		if (size == null)
			size = component.preferredSize();
// [IAN]			size = preferredSize();
		Dimension sz = new Dimension(size);
		if (sel != XFDesignConstants.SEL_UNSELECT) {
			sz.height += 2*XFDesignOptions.selectionSize;
			sz.width += 2*XFDesignOptions.selectionSize;
		}
		sz.height += borderSize;

		return sz;
	}

	public Dimension getSize() { 
		Dimension sz = component.preferredSize();
		if (autoSize) 
			size = sz;
		if (size == null)
			size = sz;
		return new Dimension(size);
	}

	public int setDesignSize(int w, int h) { 
		if (autoSize == true) {
			size = component.preferredSize();
		}
		else {
			if (size == null)
				size = new Dimension(w,h);
			else {
				size.width = w;  
				size.height = h;
			}
		}
		component.resize(size);
		regenForm(XFDesignConstants.GEN_RESHAPE);
		repaintComponent();
		return XFDesignConstants.RSP_OK;
	}

	protected abstract void regenForm(int what);

	protected boolean allowsDrag() {
		return ((!autoSize) && 
			((XelfiVisualContainer)getParent()).getDesignLayout().allowsDragging());
	}

	public void paint(Graphics g) {
		if (XFDesignOptions.designMode == XFDesignConstants.DM_REAL)
			return;
		Dimension d = size();
		g.setColor(XFDesignOptions.draggingColor);
		if (sel != XFDesignConstants.SEL_UNSELECT) {
			int i = XFDesignOptions.selectionSize;
			if (borderSize != 0)
				g.fillRect(i, i, d.width - 2*i, d.height - 2*i);
			g.setColor(XFDesignOptions.selectionColor);
			g.drawRect(i - 2, i - 2, d.width - 2*i + 3, d.height - 2*i + 3);
			if (sel == XFDesignConstants.SEL_SELECT) {
				for (int k = 0; k <= 2; k++) {
					if ((k == 1) && (d.width < 3*i +2))
						continue;
					int x = 0;
					switch (k) {
						case 1: x = (d.width-i)/2; break;
						case 2: x = d.width-i; break;
					}
					for (int l = 0; l <=2; l++) {
						if ((k == 1) && (l == 1))
							continue;
						if ((l == 1) && (d.height < 3*i +2))
							continue;
						int y = 0;
						switch (l) {
							case 1: y = (d.height-i)/2; break;
							case 2: y = d.height-i; break;
						}
						if (k*3+l+1 == dragMode) {
							g.setColor(XFDesignOptions.selectionSpecColor);
							g.fillRect(x,y,i,i);
							g.setColor(XFDesignOptions.selectionColor);
						}
						else g.fillRect(x,y,i,i);
					}
				}
			}
		}
		else {
			if (borderSize != 0)
				g.fillRect(0, 0, d.width, d.height);
		}
	}

	/**
	 * Sets whether Component is selected or not.
	 * @param aSel - true means that component is selected, false that not
	 */
	public void setSelection(int aSel) {
		if (sel == aSel)
			return;
		sel = aSel;
		repaintComponent();
		repaint();
	}

	public int getSelection() {
		return sel;
	}

	protected void repaintComponent() {
		((Component)this).invalidate();
		if (((Component)this).getParent() != null)
			((Component)this).getParent().validate();
	}
		
	public String getName() { 
		return name;
	}

	public int setDesignName(String aName) { 
		name = aName;
//[IAN 20 Feb]		regenForm(XFDesignConstants.GEN_ALL);
		return XFDesignConstants.RSP_OK;
	}

	public Point getPosition() { 
		return new Point(position.x,position.y);
	}

	public int setPosition(int x, int y) { 
		if (position == null)
			position = new Point(x, y);
		else {
			position.x = x;
			position.y = y;
		}
		regenForm(XFDesignConstants.GEN_RESHAPE);
		repaintComponent();
		return XFDesignConstants.RSP_OK;
	}

	protected boolean setAutoSize(boolean aAutoSize) {
		if (aAutoSize == autoSize) 
			return false;
		autoSize = aAutoSize;
		if (autoSize) 
			setDesignSize(0,0);
		repaintComponent();
		return autoSize;
	}

	protected Image loadIcon(String filename) {
		String sep = new String(System.getProperty("file.separator"));
		StringBuffer s = new StringBuffer(XelfiPath.get() + sep + 
			"design" + sep + "components" + sep + "xawt" + sep + filename);
		return Toolkit.getDefaultToolkit().getImage(new String(s));
	}

	public boolean handleEvent(Event evt) {
		if (XFDesignOptions.designMode==XFDesignConstants.DM_DESIGN) {
			if ((sel == XFDesignConstants.SEL_SELECT) && (evt.id == Event.MOUSE_DOWN)) {
				dragMode = resolveMode(evt.x, evt.y);
				if (dragMode != DM_NONE) {
					dragOrigin = new Point(evt.x, evt.y);
					if ((autoSize) || (!allowsDrag())) {
						dragMode = DM_NONE;
						repaint();
						return false;
					}
					repaint();
					return true;
				}
			}
			else if ((sel == XFDesignConstants.SEL_SELECT) && (evt.id == Event.MOUSE_UP)) {
				if (dragMode != DM_NONE) {
					Dimension d = getSize();
					Point p = getPosition();
					boolean changePos = false;
					if ((dragMode != DM_UM) && (dragMode != DM_DM)) {
						if ((dragMode == DM_UL) || (dragMode == DM_ML) || (dragMode == DM_DL)) {
							d.width += dragOrigin.x - evt.x;
							p.x += evt.x - dragOrigin.x;
							changePos = true;
						}
						else d.width -= dragOrigin.x - evt.x;
						if (d.width < 0) d.width = 0;
					}
					if ((dragMode != DM_ML) && (dragMode != DM_MR)) {
						if (d.height < 0)
							d.height = 0;
						if ((dragMode == DM_UL) || (dragMode == DM_UM) || (dragMode == DM_UR)) {
							p.y += evt.y - dragOrigin.y;
							d.height += dragOrigin.y - evt.y;
							changePos = true;
						}
						else d.height -= dragOrigin.y - evt.y;
						if (d.height < 0) d.height = 0;
					}
					if (changePos)
						setPosition(p.x, p.y);
					setDesignSize(d.width, d.height);
					TopLevel.getInspector().selectionChanged();
					dragMode = DM_NONE;
				}
			}
			return false;
		}
		else return super.handleEvent(evt);
	}

	protected boolean setParentColor(boolean aParentColor) {
		if (aParentColor == parentColor)
			return false;
		parentColor = aParentColor;
		if (parentColor)
			parentUpdate();
		return parentColor;
	}

	protected boolean setParentFont(boolean aParentFont) {
		if (aParentFont == parentFont)
			return false;
		parentFont = aParentFont;
		if (parentFont)
			parentUpdate();
		return parentFont;
	}
	
	public boolean parentUpdate() {
		boolean b = false;
//		if ((parentFont) && (!getParent().getFont().equals(component.getFont()))) {
		if (parentFont) {
			b = true;
			component.setFont(getParent().getFont());
			if (autoSize)
				size = component.preferredSize();
		}
		if (parentColor) {
//			if (!getParent().getForeground().equals(component.getForeground())) {
				b = true;
				component.setForeground(getParent().getForeground());
//			}
//			if (!getParent().getBackground().equals(component.getBackground())) {
//				b = true;
				component.setBackground(getParent().getBackground());
//			}
		}
		if (b) {
//			component.hide();
//			component.show();
			component.invalidate();
//			repaintComponent();
		}
//		regenForm(XFDesignConstants.GEN_SETPROPS);
		return b;
	}

	private int resolveMode(int x, int y) {
		int res = 0;
		int df = XFDesignOptions.selectionSize;
		Dimension d = size();

		if (x < df)
			res = 0;
		else if (x >= d.width - df)
			res = 6;
		else if ((x >= (d.width-df)/2) && (x < (d.width-df)/2+df) && (d.width >= 3*df+2))
			res = 3;
		else return DM_NONE;
		
		if (y <= df)
			res += 1;
		else if (y >= d.height - df)
			res += 3;
		else if ((y >= (d.height-df)/2) && (y < (d.height-df)/2+df) && (d.height >= 3*df+2))
			res += 2;
		else return DM_NONE;
		
		if (res == 5)
			return DM_NONE;
		else
			return res;
	}

	public void loadFromStream(DataInput stream) throws	IOException {
		name = stream.readUTF();

		int x = stream.readInt();
		int y = stream.readInt();
		setPosition(x,y);
		
		int w = stream.readInt();
		int h = stream.readInt();
		setDesignSize(w,h);
	}
	
	public void saveToStream(DataOutput stream) throws IOException {
		stream.writeUTF(name);
		
		Point pos = getPosition();
		stream.writeInt(pos.x);
		stream.writeInt(pos.y);

		Dimension size1 = getSize();
		stream.writeInt(size1.width);
		stream.writeInt(size1.height);
	}
}
