/**
* Xelfi
*/

package xelfi.design.components.xawt;

/**
  *
  * @version 	2.12, 03 Apr 1997
  * @author 	Petr Hamernik, Ian Formanek
  *
  */

import java.awt.*;
import java.io.*;
import xelfi.design.components.*;
import xelfi.design.components.property.*;
import xelfi.design.forms.XelfiDesignForm;
import xelfi.design.util.XFDesignConstants;
import xelfi.design.util.XFDesignOptions;
import xelfi.design.util.XFDesignUtils;

public abstract class XScrollbar extends XAWTPanel {
	protected Scrollbar scrollbar;
	private static int propertyCount = 13;
	private Object propDef[];
	private boolean propDefSet = false;
	private static int eventCount = 7;
	private String events[] = new String[eventCount];

	private int orientation;
	private String orientationStr;
	
	public XScrollbar(int aOrientation, String aOrientationStr) {
		super(XAWTPanel.WITH_DRAGBORDER);
		setComponent(scrollbar = new Scrollbar(aOrientation));
		orientation = aOrientation;
		orientationStr = aOrientationStr;
		propDef = new Object[propertyCount];
		for (int i=0; i<propertyCount; i++)
			propDef[i] = null;
	}

	protected void regenForm(int what) {
		if (parentForm != null)
			parentForm.regenerateCode(this, what);
	}

	public String[] neededImports() {
		String[] ret = new String[1];
		ret[0] = "java.awt.Scrollbar";
		return ret;
	}

	private boolean propertyDefault(int i) throws XFPropertyIndexOutOfBoundsException {
		if (propDefSet)
			propertiesSetDefault();
		if (propDef[i] == null)
			return false;
		switch (i) {
			case 0: return ((Color) propDef[i]).equals(scrollbar.getBackground());
			case 1: return ((Color) propDef[i]).equals(scrollbar.getForeground());
			case 2: return (((Boolean) propDef[i]).booleanValue() == scrollbar.isEnabled());
			case 3: return (((Boolean) propDef[i]).booleanValue() == bVisible);
			case 4: return (((Boolean) propDef[i]).booleanValue() == autoSize);
			case 5: return (((Boolean) propDef[i]).booleanValue() == parentColor);
			case 6: return true;
			case 7: return (((Integer) propDef[i]).intValue() == scrollbar.getValue());
			case 8: return (((Integer) propDef[i]).intValue() == scrollbar.getMinimum());
			case 9: return (((Integer) propDef[i]).intValue() == scrollbar.getMaximum());
			case 10: return (((Integer) propDef[i]).intValue() == scrollbar.getLineIncrement());
			case 11: return (((Integer) propDef[i]).intValue() == scrollbar.getPageIncrement());
			case 12: return (((Integer) propDef[i]).intValue() == scrollbar.getVisible());
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	private void propertiesSetDefault() {
		if (propDefSet)
			return;
		propDefSet = true;
		if (scrollbar.getBackground() != null)
				propDef[0] = new Color(scrollbar.getBackground().getRGB());
		if (scrollbar.getForeground() != null)
				propDef[1] = new Color(scrollbar.getForeground().getRGB());
		propDef[2] = new Boolean(scrollbar.isEnabled());
		propDef[3] = new Boolean(bVisible);
		propDef[4] = new Boolean(autoSize);
		propDef[5] = new Boolean(parentColor);
		propDef[7] = new Integer(scrollbar.getValue());
		propDef[8] = new Integer(scrollbar.getMinimum());
		propDef[9] = new Integer(scrollbar.getMaximum());
		propDef[10] = new Integer(scrollbar.getLineIncrement());
		propDef[11] = new Integer(scrollbar.getPageIncrement());
		propDef[12] = new Integer(scrollbar.getVisible());
	}

	public int getPropertyCount() { return propertyCount; }
	
	public Object getProperty(int index) throws XFPropertyIndexOutOfBoundsException { 
		switch (index) {
			case 0: return scrollbar.getBackground();
			case 1: return scrollbar.getForeground();
			case 2: return (scrollbar.isEnabled()) ? Boolean.TRUE : Boolean.FALSE;
			case 3: return (bVisible) ? Boolean.TRUE : Boolean.FALSE;
			case 4: return (autoSize) ? Boolean.TRUE : Boolean.FALSE;
			case 5: return (parentColor) ? Boolean.TRUE : Boolean.FALSE;
			case 6: return orientationStr;
			case 7: return new Integer(scrollbar.getValue());
			case 8: return new Integer(scrollbar.getMinimum());
			case 9: return new Integer(scrollbar.getMaximum());
			case 10: return new Integer(scrollbar.getLineIncrement());
			case 11: return new Integer(scrollbar.getPageIncrement());
			case 12: return new Integer(scrollbar.getVisible());
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}
			
	public String[] getPropertyNames() { 
		String list[] = new String[propertyCount];
		list[0] = "Background Color";
		list[1] = "Foreground Color";
		list[2] = "Enabled";
		list[3] = "Visible";
		list[4] = "Auto Size";
		list[5] = "Parent Color";
		list[6] = "Orientation";
		list[7] = "Value";
		list[8] = "Minimum";
		list[9] = "Maximum";
		list[10] = "Line Increment";
		list[11] = "Page Increment";
		list[12] = "Visible Amount";
		return list;
	}
		
	public String getPropertyString(int index) throws 
		XFPropertyIndexOutOfBoundsException { 
		switch (index) {
			case 0: 
			case 1: return XColorPropertyType.viewString(getProperty(index));
			case 2:
			case 3: return XBooleanPropertyType.viewString(getProperty(index));
			case 4: 
			case 5: return XBooleanPropertyType.viewString(getProperty(index));
			case 6: return orientationStr;
			case 7:
			case 8:
			case 9:
			case 10:
			case 11:
			case 12: return XIntegerPropertyType.viewString(getProperty(index));
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public int getPropertyViewerType(int index) throws 
		XFPropertyIndexOutOfBoundsException {
		switch (index) {
			case 0: 
			case 1: return XFDesignConstants.PVT_SPECIAL;
			case 2:
			case 3:
			case 4: 
			case 5: return XFDesignConstants.PVT_CHECK;
			case 6: return XFDesignConstants.PVT_READONLY;
			case 7:
			case 8:
			case 9:
			case 10:
			case 11:
			case 12: return XFDesignConstants.PVT_INPUT_INT;
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public String[] getPropertyValueNames(int index) throws 
		XFPropertyIndexOutOfBoundsException {
		switch (index) {
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public Object[] getPropertyValueValues(int index) throws 
		XFPropertyIndexOutOfBoundsException {
		switch (index) {
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public int setProperty(int index, Object value) throws 	
		XFPropertyIndexOutOfBoundsException,
		XFPropertyBadValueTypeException 
	{ 
		int ret = XFDesignConstants.RSP_OK;
		int regen = XFDesignConstants.GEN_SETPROPS;
		if (!propDefSet)
			propertiesSetDefault();
		switch (index) { 
			case 0: if (!(value instanceof Color)) throw new XFPropertyBadValueTypeException();
					scrollbar.setBackground((Color)value);
					ret = ret | XFDesignConstants.RSP_REPAINT_COMP;
					scrollbar.hide();
					scrollbar.show();
					if (parentColor) {
						parentColor = false;
						ret = ret | XFDesignConstants.RSP_OTHER_PROP;
					}
					break;
			case 1: if (!(value instanceof Color)) throw new XFPropertyBadValueTypeException();
					scrollbar.setForeground((Color)value);
					ret = ret | XFDesignConstants.RSP_REPAINT_COMP;
					scrollbar.hide();
					scrollbar.show();
					if (parentColor) {
						parentColor = false;
						ret = ret | XFDesignConstants.RSP_OTHER_PROP;
					}
					break;
			case 2: if (!(value instanceof Boolean)) throw new XFPropertyBadValueTypeException();
					scrollbar.enable(((Boolean)value).booleanValue());
					break;
			case 3: if (!(value instanceof Boolean)) throw new XFPropertyBadValueTypeException();
					bVisible = ((Boolean)value).booleanValue();
					break;
			case 4: if (!(value instanceof Boolean)) throw new XFPropertyBadValueTypeException();
					setAutoSize(((Boolean)value).booleanValue());
					if (autoSize) {
						repaintComponent();
						setDesignSize(0,0);
						ret = ret | XFDesignConstants.RSP_OTHER_PROP;
					}
					regen = regen | XFDesignConstants.GEN_RESHAPE;
					ret = ret | XFDesignConstants.RSP_REPAINT_COMP;
					break;
			case 5: if (!(value instanceof Boolean)) throw new XFPropertyBadValueTypeException();
					setParentColor(((Boolean)value).booleanValue());
					if (parentColor)
						ret = ret | XFDesignConstants.RSP_REPAINT_COMP;
					break;
			case 7:	if (!(value instanceof Integer)) throw new XFPropertyBadValueTypeException();
					scrollbar.setValue(((Integer)value).intValue());
					ret = ret | XFDesignConstants.RSP_OTHER_PROP;
					break;
			case 8:	if (!(value instanceof Integer)) throw new XFPropertyBadValueTypeException();
					scrollbar.setValues(scrollbar.getValue(),scrollbar.getVisible(),
						((Integer)value).intValue(), scrollbar.getMaximum());
					ret = ret | XFDesignConstants.RSP_OTHER_PROP;
					break;
			case 9:	if (!(value instanceof Integer)) throw new XFPropertyBadValueTypeException();
					scrollbar.setValues(scrollbar.getValue(),scrollbar.getVisible(),
						scrollbar.getMinimum(), ((Integer)value).intValue());
					ret = ret | XFDesignConstants.RSP_OTHER_PROP;
					break;
			case 10:if (!(value instanceof Integer)) throw new XFPropertyBadValueTypeException();
					scrollbar.setLineIncrement(((Integer)value).intValue());
					ret = ret | XFDesignConstants.RSP_OTHER_PROP;
					break;
			case 11:if (!(value instanceof Integer)) throw new XFPropertyBadValueTypeException();
					scrollbar.setPageIncrement(((Integer)value).intValue());
					ret = ret | XFDesignConstants.RSP_OTHER_PROP;
					break;
			case 12:if (!(value instanceof Integer)) throw new XFPropertyBadValueTypeException();
					scrollbar.setValues(scrollbar.getValue(),((Integer)value).intValue(), 
						scrollbar.getMinimum(), scrollbar.getMaximum());
					break;

			default: throw new XFPropertyIndexOutOfBoundsException();
		}
		if (parentForm != null)
			parentForm.regenerateCode(this, regen);
		
		return ret;
	}
			
	public void specialPropertyInput(Frame parent, int index) throws 	
		XFPropertyIndexOutOfBoundsException, 
		XFPropertyNoSpecialInputException { 
		if ((index < 0) || (index >= propertyCount)) throw new XFPropertyIndexOutOfBoundsException();
		switch (index) { 
			case 0:
			case 1: XColorPropertyType.invokeSpecialInput(parent, this, index);
						break;
			default: throw new XFPropertyNoSpecialInputException();
		}
	}

	public int getEventCount() {
		return eventCount;
	}

	public String[] getEventNames() {
		String list[] = new String[eventCount];
		list[0] = "OnGotFocus";
		list[1] = "OnLostFocus";
		list[2] = "OnScrollAbolute";
		list[3] = "OnScrollLineDown";
		list[4] = "OnScrollLineUp";
		list[5] = "OnScrollPageDown";
		list[6] = "OnScrollPageUp";
		return list;
	}

	public String getEvent(int index) throws 
		XFPropertyIndexOutOfBoundsException {
		if ((index>=0) && (index<eventCount)) {
			if (events[index] == null)
				return null;
			else
				return new String(events[index]); 
		}
		else throw new XFPropertyIndexOutOfBoundsException();
	}		
		
	public void setEvent(int index, String value) throws 
		XFPropertyIndexOutOfBoundsException {
		if ((index>=0) && (index<eventCount))
			events[index] = value;
		else throw new XFPropertyIndexOutOfBoundsException();
	}		

	public String getClassName() {
		return "Scrollbar";
	}
	
	public String generateCode(int what) {
		String gen = "";
		switch (what) {
			case XFDesignConstants.GEN_VARIABLE: 
				gen = getClassName()+" "+ name + ";\n";
				break;
			case XFDesignConstants.GEN_INIT: 
				gen = name + " = new " + getClassName() + "(Scrollbar." + orientationStr + ");\n";
				break;
			case XFDesignConstants.GEN_SETPROPS:
				if (!propDefSet)
					propertiesSetDefault();
				for (int i = 0; i < propertyCount; i++) {
					if (!propertyDefault(i)) {
						switch(i) {
							case 0: if (!parentColor)
										gen = gen + name + ".setBackground(" +
											XFDesignUtils.genColor(scrollbar.getBackground()) + ");\n";
									break;
							case 1: if (!parentColor)
										gen = gen + name + ".setForeground(" +
											XFDesignUtils.genColor(scrollbar.getForeground()) + ");\n";
									break;
							case 2: gen = gen + name + (scrollbar.isEnabled() ? ".enable();\n" : ".disable();\n");
									break;
							case 3: gen = gen + name + ((bVisible) ? ".show();\n" : ".hide();\n");
									break;
							case 7: //generate properties 7,8,9,12
									if ((!propertyDefault(8))||(!propertyDefault(9))||(!propertyDefault(12))) {
										gen = gen + name + ".setValues(" + scrollbar.getValue() + "," +
											scrollbar.getVisible() + "," + scrollbar.getMinimum() + "," +
											scrollbar.getMaximum() + ");\n";
									}
									else {
										if (!propertyDefault(8))
											gen = gen + name + ".setMinimum(" + scrollbar.getMinimum() + ");\n";
										if (!propertyDefault(9))
											gen = gen + name + ".setMaximum(" + scrollbar.getMaximum() + ");\n";
										if (!propertyDefault(12))
											gen = gen + name + ".setVisible(" + scrollbar.getVisible() + ");\n";
											
									}
									break;
							case 8:
							case 9: break;
							case 10: gen = gen + name + ".setLineIncrement(" + scrollbar.getLineIncrement() + ");\n";
							case 11: gen = gen + name + ".setPageIncrement(" + scrollbar.getPageIncrement() + ");\n";
							case 12: break;
							
						}
					}
				}
				break;			
			case XFDesignConstants.GEN_RESHAPE:
				if (((XelfiVisualContainer) getParent()).getDesignLayout().allowsDragging()) {
					if (!autoSize) 
						if (getParent() instanceof XelfiDesignForm)
							gen = name + ".reshape(insets.left+"+position.x+", insets.top+"+position.y+","+size.width+","+size.height+");\n";
						else 
							gen = name + ".reshape("+position.x+","+position.y+","+size.width+","+size.height+");\n";
					else {
						gen = name + ".resize(" + name + ".preferredSize());\n";
						if (getParent() instanceof XelfiDesignForm)
							gen = gen + name + ".move(insets.left+"+position.x+", insets.top+"+position.y+");\n";
						else
							gen = gen + name + ".move("+position.x+","+position.y+");\n";
					}
				}
				break;
		}
		return gen;
	}

	public String generateEventCondition(int index) throws XFPropertyIndexOutOfBoundsException {
		String evt = XFDesignOptions.eventVarName;
		switch (index) { 
			case 0: return evt +".id == Event.GOT_FOCUS && "+evt+".target == "+getName();
			case 1: return evt +".id == Event.LOST_FOCUS && "+evt+".target == "+getName();
			case 2: return evt +".id == Event.SCROLL_ABSOLUTE && "+evt+".target == "+getName();
			case 3: return evt +".id == Event.SCROLL_LINE_DOWN && "+evt+".target == "+getName();
			case 4: return evt +".id == Event.SCROLL_LINE_UP && "+evt+".target == "+getName();
			case 5: return evt +".id == Event.SCROLL_PAGE_DOWN && "+evt+".target == "+getName();
			case 6: return evt +".id == Event.SCROLL_PAGE_UP && "+evt+".target == "+getName();
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}
	
	public void loadFromStream(DataInput stream) throws IOException {

		super.loadFromStream(stream);

		if (!propDefSet)
			propertiesSetDefault();

		for (int i = 0; i < propertyCount; i++) {
			if (stream.readBoolean()) {
				propDef[i] = null;
			}
			else switch (i) {
				case 0: scrollbar.setBackground(XColorPropertyType.loadFromStream(stream)); break;
				case 1: scrollbar.setForeground(XColorPropertyType.loadFromStream(stream)); break;
				case 2: scrollbar.enable(XBooleanPropertyType.loadBFromStream(stream)); break;
				case 3: bVisible = XBooleanPropertyType.loadBFromStream(stream); break;
				case 4: setAutoSize(XBooleanPropertyType.loadBFromStream(stream)); break;
				case 5: setParentColor(XBooleanPropertyType.loadBFromStream(stream)); break;
				case 6: scrollbar.setValues(scrollbar.getValue(),scrollbar.getVisible(),
						XIntegerPropertyType.loadFromStream(stream).intValue(),scrollbar.getMaximum()); break;
				case 7: scrollbar.setValues(scrollbar.getValue(),scrollbar.getVisible(),
						scrollbar.getMinimum(),XIntegerPropertyType.loadFromStream(stream).intValue()); break;
				case 8: scrollbar.setValue(XIntegerPropertyType.loadFromStream(stream).intValue()); break;
				case 9: scrollbar.setLineIncrement(XIntegerPropertyType.loadFromStream(stream).intValue()); break;
				case 10: scrollbar.setPageIncrement(XIntegerPropertyType.loadFromStream(stream).intValue()); break;
				case 11: scrollbar.setValues(scrollbar.getValue(),XIntegerPropertyType.loadFromStream(stream).intValue(),
						scrollbar.getMinimum(),scrollbar.getMaximum()); break;
			}
		}

		for (int i = 0; i < eventCount; i++)
			if (stream.readBoolean())
				parentForm.assignEvent(this, i, stream.readUTF());
	}		
	
	public void saveToStream(DataOutput stream) throws IOException {

		super.saveToStream(stream);

		for (int i = 0; i < propertyCount; i++) {
			boolean b = propertyDefault(i);
			stream.writeBoolean(b);
			if (!b) {
				switch (i) {
					case 0: XColorPropertyType.saveToStream(scrollbar.getBackground(), stream); break;
					case 1: XColorPropertyType.saveToStream(scrollbar.getForeground(), stream); break;
					case 2: XBooleanPropertyType.saveToStream(scrollbar.isEnabled(), stream); break;
					case 3: XBooleanPropertyType.saveToStream(bVisible, stream); break;
					case 4: XBooleanPropertyType.saveToStream(autoSize, stream); break;
					case 5: XBooleanPropertyType.saveToStream(parentColor, stream); break;
					case 6: XIntegerPropertyType.saveToStream(new Integer(scrollbar.getMinimum()), stream); break;
					case 7: XIntegerPropertyType.saveToStream(new Integer(scrollbar.getMaximum()), stream); break;
					case 8: XIntegerPropertyType.saveToStream(new Integer(scrollbar.getValue()), stream); break;
					case 9: XIntegerPropertyType.saveToStream(new Integer(scrollbar.getLineIncrement()), stream); break;
					case 10: XIntegerPropertyType.saveToStream(new Integer(scrollbar.getPageIncrement()), stream); break;
					case 11: XIntegerPropertyType.saveToStream(new Integer(scrollbar.getVisible()), stream); break;
				}
			}
		}

		for (int i = 0; i < eventCount; i++) {
			if ((events[i] != null) && (events[i].length() > 0)) {
				stream.writeBoolean(true);
				stream.writeUTF(events[i]);
			}
			else stream.writeBoolean(false);
		}
	}
	
}