/**
* Xelfi
*/

package xelfi.design.forms;

import java.io.*;
import java.util.NoSuchElementException;
import xelfi.top.TopLevel;
import xelfi.editor.Source;
import xelfi.editor.DocumentHandler;
import xelfi.editor.XelfiFile;
import xelfi.templates.*;
import xelfi.util.Debug;

/**
* Basic abstract class for managing form classes.
* Provides all functionality except creating new sources from
* templates. It is done in another way.
*
* @version  1.06, 25 Mar 1997
* @author   Ian Formanek, Jaroslav Tulach
*/

abstract class FormsManager extends PlainManager {
	/** Returns design form for this manager.
	* @param t the template to generate from
	* @param s the source to generate in
	* @return the application form
	*/
	public abstract XelfiDesignForm getDesignForm (Template t, Source s)
		throws IOException;
  
	/** Does not store anything.
	* @param t the template to work with
	* @return null
	*/
	public String storeInfo (Template t) {
		return null;
	}

	/** Does not load anything.
	* @param t the template to work with
	* @param s the string to load from
	*/
	public void loadInfo (Template t, String s) throws IllegalArgumentException {
	}

	/** Fills new source from template. Calls getDesignForm to get
	* appropriate XelfiDesignForm.
	* @param t the template
	* @param s the source
	*/
	public void newSource (Template t, Source s) throws Exception {
		XelfiDesignForm form = getDesignForm (t, s);
		s.setHandler (new FormHandler (form));
		TopLevel.getInspector().inspectableChanged(form);
	}

	public Template identifySource (Source s) 
		throws NoSuchElementException,IOException
	{
		XelfiFile xf = s.getFile ();
		File dir = new File (xf.getFile ().getParent (), xf.getName () + ".form");
		if (!dir.exists ()) throw new NoSuchElementException ();

		// this can throw IOException if the form does not load properly
		XelfiDesignForm form = XFFormLoader.loadForm(dir.toString(), s);

		s.setHandler (new FormHandler (form));
		return Template.find (form.getTemplateName());
	}

	public TemplateManager createTemplate (Template old, Source s, Template theNew) throws Exception {
		DocumentHandler dh = s.getHandler();
		if (!(dh instanceof XelfiDesignForm)) {
			Debug.debug(Debug.ERROR, "STRANGE: Given Source is not a form source!!");
			return null;
		}
		File file = fileForTemplate (theNew);
		XFFormLoader.saveForm((XelfiDesignForm)dh, file.toString());
		return new FileFormsManager ();
	}
}
