/**
* Xelfi
*/

package xelfi.design.forms.layouts;

import java.io.*;
import java.awt.CardLayout;
import java.util.Vector;
import java.awt.Container;
import java.awt.Component;
import java.awt.Frame;
import xelfi.design.components.*;
import xelfi.design.util.XFDesignConstants;

/* 
 * Class XBorderLayout 
 *
 * @version 	1.05, 15 Mar 1997
 * @author 	Ian Formanek
 */

public class XCardLayout extends CardLayout implements XelfiDesignLayout {
	private Vector components=new Vector(1,1);
	private Vector cards=new Vector(1,1);
	private int propertyCount=2;

	private int showingCard = -1;
	private int lastCard = 0;

	private XelfiVisualContainer parent = null;

	public void setParent(XelfiVisualContainer aParent) {
		parent = aParent;
	}
	
	public int getPropertyCount() { return propertyCount;}

	public String[] getPropertyNames() { 
		String list[] = new String[propertyCount];
		list[0] = new String("Showing Card");
		list[1] = new String("Card Name");
		return list; 
	}

	public void addComponent(XelfiVisualComponent comp) {
		lastCard++;
		components.addElement(comp.getName());
		cards.addElement("Card"+lastCard);
	}
	
	public void renameComponent(XelfiVisualComponent comp, String newName) throws 
		XFNameNotFoundException {
		int index = components.indexOf(comp.getName());
		if (index == -1) 
			throw new XFNameNotFoundException();
		components.setElementAt(newName,index);
	}
	
	public void removeComponent(XelfiVisualComponent comp) throws 
		XFNameNotFoundException {
		String compName = comp.getName();
		int index = components.indexOf(compName);
		if (index == -1) 
			throw new XFNameNotFoundException();
		components.removeElementAt(index);
		cards.removeElementAt(index);
//		if ((index == showingCard) && components.size()>0) {
//			showingCard = 1;
//			show some other card			
	}

	
	public int getPropertyViewerType(int index) throws 
		XFPropertyIndexOutOfBoundsException {
		switch (index) {
			case 0: return XFDesignConstants.PVT_CHOICE_STRICT;
			case 1: return XFDesignConstants.PVT_INPUT_STRING;
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public String[] getPropertyValueNames(int index) throws 
		XFPropertyIndexOutOfBoundsException {
		switch (index) {
			case 0:	String list[] = new String[components.size()];
						cards.copyInto(list);
						return list;
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public Object[] getPropertyValueValues(int index) throws 
		XFPropertyIndexOutOfBoundsException {
		switch (index) {
			case 0:	int count = components.size();
						Integer list[] = new Integer[count];
						for (int i=0; i<count; i++) list[i] = new Integer(i);
						return list;
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public Object getProperty(XelfiVisualComponent comp, int index) throws 
		XFPropertyIndexOutOfBoundsException,
		XFNameNotFoundException {
		switch (index) {
			case 0:	return new Integer(showingCard);
			case 1: int idx = components.indexOf(comp.getName());
						if (idx == -1) throw new XFNameNotFoundException();
						return cards.elementAt(idx);
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}
		
	public String getPropertyString(XelfiVisualComponent comp, int index) throws 
		XFPropertyIndexOutOfBoundsException,
		XFNameNotFoundException {
		switch (index) {
			case 0:	if (showingCard == -1) return "none";
						else return (String)cards.elementAt(showingCard);
			case 1: int idx = components.indexOf(comp.getName());
						if (idx == -1) throw new XFNameNotFoundException();
						return (String)cards.elementAt(idx);
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}
		
	public int setProperty(XelfiVisualComponent comp, int index, Object value) throws 
		XFPropertyIndexOutOfBoundsException,
		XFPropertyBadValueTypeException,
		XFNameNotFoundException {
		int ret = XFDesignConstants.RSP_OK;
		switch (index) {
			case 0: if (!(value instanceof Integer)) throw new XFPropertyBadValueTypeException();
						int sc = ((Integer)value).intValue();
						if (sc<0 || sc>=components.size()) return XFDesignConstants.RSP_DIFF_VALUE;
						showingCard = sc;
						show(((Component)comp).getParent(),(String)cards.elementAt(showingCard));
						break;
			case 1: if (!(value instanceof String)) throw new XFPropertyBadValueTypeException();
						int idx = components.indexOf(comp.getName());
						String sValue = (String)value;
						if (idx == -1) throw new XFNameNotFoundException();
						if (sValue.compareTo((String)cards.elementAt(idx)) != 0) // if the new name is not same as the old one
							if (cards.indexOf(sValue) ==-1) { // if no other card has such name
								cards.setElementAt(sValue, idx);
								removeLayoutComponent((Component)comp);
								addLayoutComponent(sValue, (Component)comp);
							}
							else	ret = XFDesignConstants.RSP_DIFF_VALUE;
						break;

			default: throw new XFPropertyIndexOutOfBoundsException();
		}
		return ret;
	}

	public void specialPropertyInput(Frame parent, XelfiVisualComponent comp, int index) throws 
		XFPropertyIndexOutOfBoundsException,
		XFPropertyNoSpecialInputException,
		XFNameNotFoundException {
			if (index<0 || index >=propertyCount) throw new XFPropertyIndexOutOfBoundsException();
			else throw new XFPropertyNoSpecialInputException();
	}
		
	public String getClassName() {
		return "CardLayout";
	}

	///////////////////
	// Layout things //
	///////////////////

	public boolean allowsDragging() { return false; }

	public void layoutAdd(Container cont, XelfiVisualComponent comp) {
		String compName = comp.getName();

		int idx = components.indexOf(compName);
		if (idx == -1) {
			addComponent(comp);
			idx = components.indexOf(compName);
		}
		String cardName = (String)cards.elementAt(idx);

		if (showingCard == idx) { // we finish loadingFromStream by showing this card
			if (parent != null) {
				cont.add(cardName, (Component)comp);
				show((Container)parent,cardName);
			}
		}
		else {
			if (showingCard == -1) showingCard = idx; // [PENDING - will it appear in CI?]
			cont.add(cardName, (Component)comp);
		}

	}

	public void layoutAdd(Container cont, int pos, XelfiVisualComponent comp) {
		layoutAdd(cont, comp);
	}

	/////////////////////
	// Code generation //
	/////////////////////
	
	public String[] neededImports() {
		String list[] = new String[0];
		return list;
	}

	// this method returns whether it needs a variable for the layout
	public boolean genLayoutVariable() { return false; }
	
	public String genLayoutInitCode(String containerName, String varName) {
		if (containerName == null)
			return "setLayout(new CardLayout());\n";
		else 
			return containerName+".setLayout(new CardLayout());\n";
	}
	
	public String genComponentLayoutCode(String containerName, XelfiVisualComponent comp, String varName) {
		String compName = comp.getName();
		int idx = components.indexOf(compName);
		if (idx == -1) return ""; // component not found
		String cardName = (String)cards.elementAt(idx);
		if (cardName == null) return null;
		if (containerName == null)
			return "add(\""+cardName+"\","+compName+");";
		else 
			return containerName+".add(\""+cardName+"\","+compName+");";
	}
	/////////////////
	// Persistence //
	/////////////////

	public void loadFromStream(DataInput stream) throws
		IOException { 
		int count = stream.readInt();
		showingCard = stream.readInt();
		lastCard = stream.readInt();
		for (int i= 0; i<count; i++) {
			components.addElement(stream.readUTF());
			cards.addElement(stream.readUTF());
		}
	}
	
	public void saveToStream(DataOutput stream) throws
		IOException { 
		int count = components.size();
		stream.writeInt(count);
		stream.writeInt(showingCard);
		stream.writeInt(lastCard);
		for (int i= 0; i<count; i++) {
			stream.writeUTF((String)components.elementAt(i));
			stream.writeUTF((String)cards.elementAt(i));
		}
	}
}	