package xelfi.design.inspector;

import java.awt.*;

import xelfi.util.Debug;

/**
 * Private class XFPropertyStringBox implements functionality and look
 * of one string property element. It can be used to show property names,
 * event names and values of all kind of properties when they're not 
 * edited.
 *
 * @author Dafe
 * @version 1.02, 21 Dec 1996
	Last change:  DF   24 Dec 96    3:08 pm
*/
class XFPropertyStringBox {
  private int                 index;
  private XFStringBox         stringBox;
  private XFPlasticDecorator  border;
  private Font                defaultFont = 
                                new Font("Dialog", Font.PLAIN, 10);

	XFPropertyStringBox (int index) {
		this(index, null, Color.black, null);
	}

	XFPropertyStringBox	(int index, String text) {
		this(index, text, Color.black, null);
	}

	XFPropertyStringBox	(int index, String text, Color color) {
		this(index, text, color, null);
	}

	XFPropertyStringBox	(int index, String text, Color color, Font font) {
		super();
		this.index = index;
    stringBox = new XFStringBox(text, color, 
                                font==null ? defaultFont : font);
    border = new XFPlasticDecorator();
	}

	void setText (String text) {
    stringBox.setText(text);
	}

	String getText () {
		return stringBox.getText();
	}

	void setColor (Color color) {
    stringBox.setColor(color);
	}
 
	Color getColor () {
    return stringBox.getColor();
  }

	int getIndex () {
		return index;
	}

	void setFont (Font font) {
    stringBox.setFont(font);
	}

  void up () {
    select(false);
  }

  void down () {
    select(true);
  }

	Dimension preferredSize () {
    Dimension boxSize = stringBox.preferredSize();
		return new Dimension(boxSize.width+2*border.getWidth(),
                         boxSize.height+2*border.getWidth());
	}	

  boolean isSelected () {
    return border.getType()==XFPlasticDecorator.PLASTIC_DOWN;
  }

  void toggleSelect () {
    select(!isSelected());
  }

  void select (boolean state) {
    border.setType(state ? XFPlasticDecorator.PLASTIC_DOWN
                         : XFPlasticDecorator.PLASTIC_UP
                  );
  }

	/**
	 * Paints string box - transfers real work to border and string box.
	*/
	public void paint (Graphics g, int x, int y, int width, int height) {
    border.paint(g, x, y, width, height);
    int borderWidth = border.getWidth();
    stringBox.paint(g, x+borderWidth, y+borderWidth, 
                    width-2*borderWidth, height-2*borderWidth
                   );
	}

  public void paint (Graphics g, Rectangle paintRect) {
    paint(g, paintRect.x, paintRect.y, paintRect.width, paintRect.height);
  }
}
