package xelfi.design.inspector;

import java.awt.*;

import xelfi.design.util.XFDesignConstants;
import xelfi.util.Debug;

/**
 * Private class XFValuesPanel implements functionality and look
 * of column of property value boxes.
 * 
 * @author Dafe
 * @version 1.00, 26 Nov 1996
	Last change:  DF   29 Jan 97    8:48 pm
*/
class XFValuesPanel extends XFNamesPanel {
	// public event constants
	public static final int				PROPERTY_CHANGED = 4855+2;
	public static final int				DONE = 4855+3;
	public static final int				ESCAPE = 4855+4;
	public static final int				SPECIAL_DIALOG = 4855+5;

	// possible controls that can be showed in this panel
  // (property value editing)
	private Choice								choice = null;
	private Checkbox							checkbox = null;
	private Button								specialInput = null;
	private TextField							textField = null;
	private Button								choiceButton = null;

  private Point                 oldStarting = null;

  private XFCommonProperties    commonProps;

	XFValuesPanel (XFCommonProperties commonProps) {
		super();
    this.commonProps = commonProps;
    initControls();
    setTextFieldFont(new Font("Helvetica", Font.PLAIN, 10));
	}

  public boolean mouseDown (Event e, int x, int y) {
//	  Debug.debug(Debug.INSPECTOR, "x is "+x+" y is "+y);
//    super.mouseDown(e, x, y);
		// catch mouse click on Property string box - when I'm in view mode
		// modify this message so that it acts like when value box click occurs
    XFPropertyStringBox found = findClickedPropertyBox(x, y);
    if ( found==null ) return false;
		e.id = PROPERTY_CHANGED;
		e.arg = found;
    return false;
  }

  public boolean keyDown (Event e, int key) {
    super.keyDown(e, key);
    switch ( key )
    {
      case 10: // Enter pressed
				e.id = DONE;
				e.arg = selected;
        break;
      case 27: // Esc pressed
				e.id = ESCAPE;
				e.arg = selected;
        break;
    }
    return false;
  }

	public boolean handleEvent (Event e) {
    boolean result = super.handleEvent(e);
		switch (e.id) {
			case Event.ACTION_EVENT:
				if ((specialInput!=null) && (e.target==specialInput)) {
					// special input button was activated
					e.id = SPECIAL_DIALOG;
          e.arg = selected;
        }
        else {
          if (((checkbox!=null) && (e.target==checkbox)) ||
          		((choice!=null) && (e.target==choice)) ||
          		((specialInput!=null) && (e.target==specialInput))) {
          	// checkbox clicked - modify message to DONE editing
          	// and same for choices, special buttons
          	e.id = DONE;
          	e.arg = selected;
          }                  
          else {
            if ( e.target==choiceButton ) {
              // choice button pushed = show strict choice
              changeToViewMode();
              changeToEditMode((XFPropertyValueBox)selected,
                               XFDesignConstants.PVT_CHOICE_STRICT);
            }
          }
        }
		}
		return result;
	}


  /**
   * Overrides XFNamesPanel's paint method.
   * Adds repainting (moving) of visible input controls
  */
  public void paint (Graphics g) {
    super.paint(g);
    // move all visible controls to new location
    moveControls();
  }

  /**
   * Asks all elements to refresh their content and forces repaint.
  */
  void refresh () {
    for (int i=0; i<propBoxes.size(); i++) {
      ((XFPropertyValueBox)propBoxes.elementAt(i)).refresh();
    }
    // force repainting
    repaint();
  }
  
	void changeToViewMode () {
    // hide components except choice...choice must be removed
    for (int i=0; i<countComponents(); i++) {
      Component comp = getComponent(i);
      if ( getComponent(i) instanceof Choice )
        remove(comp);
      else {
        comp.hide();
			}
    }
		repaint();
	}

  void changeToEditMode (XFPropertyValueBox target, int viewerType) {
//    repaint();
    int index = propBoxes.indexOf(target);
    Point upperLeft = calculateUpperLeft(index);
    Dimension prefSize = target.preferredSize();
    Dimension mySize = size();

		// has property special input ??
		if ((viewerType & XFDesignConstants.PVT_SPECIAL)!=0) {
			showSpecial(upperLeft.x, upperLeft.y, mySize.width, prefSize.height);
		}
		// add other controls
		switch (viewerType) {
			case XFDesignConstants.PVT_INPUT_INT:
			case XFDesignConstants.PVT_INPUT_FLOAT:
			case XFDesignConstants.PVT_INPUT_CHAR:
				showTextField(upperLeft.x, upperLeft.y, mySize.width, prefSize.height,
                      target.getCommonProps().
                      getPropertiesString(target.getIndex()));
				break;
			case XFDesignConstants.PVT_INPUT_STRING:
				String stringValue = (String)target.getCommonProps().
							 								getPropertiesValue(target.getIndex(), viewerType);
				stringValue = stringValue==null ? "" : stringValue;
				showTextField(upperLeft.x, upperLeft.y, 
											mySize.width, prefSize.height, stringValue);
				break;
			case XFDesignConstants.PVT_CHECK:
        showCheckbox(upperLeft.x, upperLeft.y, mySize.width, prefSize.height,
                     (Boolean)target.getCommonProps().
                     getPropertiesValue(target.getIndex(), viewerType));
				break;
			case XFDesignConstants.PVT_CHOICE_STRICT:
			case XFDesignConstants.PVT_CHOICE_FREE:
				String[] constNames = target.getPropertyValueNames();
        if ( viewerType==XFDesignConstants.PVT_CHOICE_STRICT )
          showStrictChoice(upperLeft.x, upperLeft.y,
                           mySize.width, prefSize.height, constNames,
                           target.getCommonProps().
                            getPropertiesString(target.getIndex()));
        else
          showFreeChoice(upperLeft.x, upperLeft.y,
                         mySize.width, prefSize.height, constNames);
				break;
		}
		repaint();
	}

	/**
	 * Sets new value into selected component
   * Does nothing if no box is selected
	*/
  boolean saveValue () {
		boolean result = false;
    if ( selected==null ) return result;
    result = setValue((XFPropertyValueBox)selected);
    if (result) 
		  ((XFPropertyValueBox)selected).refresh();
		return result;
  }

  private void showSpecial(int x, int y, int width, int height) {
    Dimension prefSize = specialInput.preferredSize();
    // reshape button to right side
    specialInput.reshape(x+width-prefSize.width, y, prefSize.width, height);
    // show button
    specialInput.show();
    specialInput.requestFocus();
  }

  private void showTextField(int x, int y, int width, int height,
                             String text) {
    Dimension prefSize = textField.preferredSize();
    textField.setText(text==null ? "" : text);
    textField.selectAll();
    // reshape textfield to whole width, leaving preferred height
    // (it's important for user to see what is he editing :-)
    int diff = (height-prefSize.height)/2;
    textField.reshape(x, y+diff, width, prefSize.height);
    // show textfield
    textField.show();
    textField.requestFocus();
  }

  private void showCheckbox(int x, int y, int width, int height,
                            Boolean state) {
    Dimension prefSize = checkbox.preferredSize();
    checkbox.setState(state==null ? false : state.booleanValue());
    // reshape checkbox to the center, leaving preferred height
    int diff = (height-prefSize.height)/2;
    checkbox.reshape(x+(width-prefSize.width)/2, y+diff,
                          prefSize.width, prefSize.height);
    // show checkbox
    checkbox.show();
    checkbox.requestFocus();
  }

  private void showStrictChoice(int x, int y, int width, int height,
                                String[] items, String selected) {
    choice = new Choice();
    // fill choice
		for (int i=0; i<items.length; i++)
			choice.addItem(items[i]);
    choice.hide();
		// JDK 1.0.2 win95 choice bug workaround.....
		// (there have to be at least two items..)
		int itemCount = choice.countItems();
		choice.enable(itemCount!=0);
		if (itemCount<2) {
			// wheee, bad hack :-(
			choice.addItem("");
		}
    // get current item nad select it
    choice.select(selected);
    add(choice);
    Dimension prefSize = choice.preferredSize();
    // reshape choice to whole width, leaving preferred height
    int diff = (height-prefSize.height)/2;
    choice.reshape(x, y+diff, width, prefSize.height);
    // show choice
    choice.show();
    choice.requestFocus();
  }

  private void showFreeChoice(int x, int y, int width, int height,
                              String[] items) {
/*    choice = new Choice();
    // fill choice
		// [PENDING] - doesn't select right item
		// (i'll have to get current item and set it)
		for (int i=0; i<items.length; i++)
			choice.addItem(items[i]);
    choice.hide();
    add(choice);          */
    textField.setText("");
    // reshape choice to minimum width, leaving preferred height
    // and let textfield to fill the rest
    Dimension prefSize = choiceButton.preferredSize();
    Dimension editPrefSize = textField.preferredSize();
    int ourHeight = Math.max(prefSize.height, editPrefSize.height);
    int choiceWidth = prefSize.width;
    int diff = (height-ourHeight)/2;
    textField.reshape(x, y+diff, width-choiceWidth, ourHeight);
    choiceButton.reshape(x+width-choiceWidth, y+diff, choiceWidth, ourHeight);
    // show choice and edit control
    textField.show();
    choiceButton.show();
    textField.requestFocus();
  }

  /**
   * Retrieve new value from user control and set it
   * to the property specified by property box.
   * Calls property box's own method for setting.
  */
	private boolean setValue (XFPropertyValueBox propertyBox) {
		// [PENDING] here should be control method that will check
		// if property is really that type as it claims in getViewerType
		// send an error message to error output if not
    int viewerType = propertyBox.getViewerType();
		// switch according to viewer type
		switch (viewerType) {
			case XFDesignConstants.PVT_INPUT_INT:
				// try to retrieve integer from text field
				Integer newInt;
				try {
					newInt = new Integer(textField.getText());
				} catch (NumberFormatException e) {
					// wrong format of value - old value isn't changed
	 				// System.err.println("CI: Error setting integer property...");
					textField.selectAll();
					return false;
				}
				propertyBox.doSetProperty(newInt);
				break;
			case XFDesignConstants.PVT_INPUT_FLOAT:
				// try to retrieve float from text field
				Double newDouble;
				try {
					newDouble = new Double(textField.getText());
				} catch (NumberFormatException e) {
					// wrong format of value - old value isn't changed
					// System.err.println("CI: Error setting float property...");
					textField.selectAll();
					return false;
				}
				propertyBox.doSetProperty(newDouble);
				break;
			case XFDesignConstants.PVT_INPUT_STRING:
				propertyBox.doSetProperty(textField.getText());
				break;
			case XFDesignConstants.PVT_CHECK:
				propertyBox.doSetProperty(new Boolean(checkbox.getState()));
				break;
			case XFDesignConstants.PVT_CHOICE_FREE:
        // here we need to recognize if user is typing freely
        // or if is he(she) choosing from choice
        if ( choiceButton.isVisible() ) {
          // oh yeah, user is typing freely in textfield
          // treat value as a string
  				propertyBox.doSetProperty(textField.getText());
          break;
				}
			case XFDesignConstants.PVT_CHOICE_STRICT:
				if (choice.getSelectedItem().compareTo("")==0) {
					// don't allow selecting of helper item
					return false;
				}
				propertyBox.doSetProperty(new Integer(choice.getSelectedIndex()));
				break;
			case XFDesignConstants.PVT_INPUT_CHAR:
        String result = textField.getText();
/*        if (result.compareTo("")==0) {
          // textField is empty - we'll try to clear echo char
          char[] tempArray = new char[1];
          tempArray[0] = (char)0;
          result = new String(tempArray);
        } */
				// don't allow property to change if it is not a single char
        if (result.length()!=1) {
          // System.err.println("CI: Error setting char property...");
					textField.selectAll();
          return false;
        }
				propertyBox.doSetProperty(new Character(result.charAt(0)));
				break;
		}
		return true;
	}

  private void initControls () {
    // create controls (except choice - must be created everytime)
  	textField = new TextField("");
    checkbox = new Checkbox();
    specialInput = new Button("...");
    choiceButton = new Button("v");
    // and hide them initially
    textField.hide();
    checkbox.hide();
    specialInput.hide();
    choiceButton.hide();
    // and add them to the panel
    add(textField);
    add(checkbox);
    add(specialInput);
    add(choiceButton);
  }

  private void setTextFieldFont (Font font) {
    if (textField==null ) return;
    textField.setFont(font);
  }

  /**
   * Moves all controls to new locations to keep their
   * positions synchronized with starting point value
  */
  void moveControls () {
    if (selected==null)	{
      oldStarting = null;
//			Debug.debug(Debug.INSPECTOR, "Selected is null....");
      return;
    }
    // compute a difference between past and
    // actual positions
    int xDiff = 0;
    int yDiff = 0;
    if (oldStarting!=null) {
      xDiff = (startingPoint.x - oldStarting.x)*getBoxHeight();
      yDiff = (startingPoint.y - oldStarting.y)*getBoxHeight();
    } 
    // update old position
    oldStarting = new Point(startingPoint.x, startingPoint.y);
//		Debug.debug(Debug.INSPECTOR, "Old starting is: "+oldStarting.toString());
    if ((xDiff==0)&&(yDiff==0)) {
      // no move needed
      return;
    }
    // move all visible components
    for (int i=0; i<countComponents(); i++) {
      Component curComp = getComponent(i);
      if (curComp.isVisible()) {
        Point pastPos = curComp.location();
//				Debug.debug(Debug.INSPECTOR, "Location was: "+pastPos.toString());
        curComp.move(pastPos.x-xDiff, pastPos.y-yDiff);
//				Debug.debug(Debug.INSPECTOR, "DIff: "+xDiff+" "+yDiff);
      }
    }
  }

  /**
   * Returns first visible component or
   * null when no such exist.
  */
  private Component getFirstVisible () {
    for (int i=0; i<countComponents(); i++) {
      Component curComp = getComponent(i);
      if (curComp.isVisible()) return curComp;
    }
    return null;
  }

	/**
	 * Sets default event name of the selected property box
	 * to the textfield.
	 * Note: We have to be in events page
	*/
	void setDefaultEventName (XFPropertyValueBox box) {
		if (box==null) return;
//		Debug.debug(Debug.INSPECTOR, "jsem tuuu");
		String eventName = ((XFPropertyValueBox)box).getDefaultEventName();
		if (eventName==null) return;
		textField.setText(eventName);
		textField.requestFocus();
		textField.selectAll();
	}

}


