package xelfi.execution;

import java.awt.*;
import java.io.*;

import xelfi.top.visual.*;
import xelfi.top.TopDialog;
import xelfi.util.MessageBox;

public class SubprocessIOWindow extends TopDialog
{
  private Process sub;
  private OutputTextArea stdoutTXA = null, stderrTXA = null;
  private PipedInputLine stdinIL = null;
  private Button kill;
  private boolean running;
  private Component outputFinishedNotificator;
  private OutputStream toClose;    // close process stdin
  private String titleWhenFinished;

  public SubprocessIOWindow(Process subProcess)
  {
    this(subProcess, "Running", "Finished.");
  }

  public SubprocessIOWindow(Process subProcess, String titleRunning, String titleFinished)
  {
    super(titleRunning, false);
    titleWhenFinished = titleFinished;
    sub = subProcess;

    boolean together = RunOptions.stdOutErrTogether();
    boolean input = RunOptions.showStdin();
      // stdout
    Thread stdoutTh = new Thread(new StreamFormatPipe(sub.getInputStream(),
      stdoutTXA = new OutputTextArea(RunOptions.clearOutputAfter())));
    stdoutTh.setPriority(Thread.MIN_PRIORITY);
      // stderr
    Thread stderrTh;
    if(!together)
      stderrTh = new Thread(new StreamFormatPipe(sub.getErrorStream(),
        stderrTXA = new OutputTextArea(RunOptions.clearOutputAfter())));
    else stderrTh = new Thread(new StreamFormatPipe(sub.getErrorStream(), stdoutTXA));
    stderrTh.setPriority(Thread.MIN_PRIORITY);

      // stdin
    if(input) stdinIL = new PipedInputLine(toClose = sub.getOutputStream());

    RowLayout mainl = new RowLayout();
    setLayout(mainl);
    RowLayoutProperties labp = new RowLayoutProperties(new Insets(4, 4, 2, 4));
    RowLayoutProperties tap = new RowLayoutProperties(new Insets(0, 4, 6, 4), 0.47f);
      // text area
    if(together)
    {
      Label lab = new Label("Stdout/StdErr", Label.CENTER);
      add(lab);
      mainl.add(lab, labp);
      add(stdoutTXA);
      if(!input) tap.compInsets.bottom = 0;
      tap.compGrowth = 0.94f;
      mainl.add(stdoutTXA, tap);
    }
    else
    {
      Label lab1 = new Label("Stdout", Label.CENTER);
      Label lab2 = new Label("Stderr", Label.CENTER);
      add(lab1);
      add(lab2);
      add(stdoutTXA);
      add(stderrTXA);
      mainl.add(lab1, labp);
      mainl.add(stdoutTXA, tap);
      mainl.add(lab2, labp);
      if(!input) tap.compInsets.bottom = 0;
      mainl.add(stderrTXA, tap);
    }
      // input line
    if(input)
    {
      Label lab = new Label("Stdin", Label.CENTER);
      add(lab);
      mainl.add(lab, labp);
      RowLayoutProperties ilp = new RowLayoutProperties(new Insets(0, 4, 0, 4));
      add(stdinIL);
      mainl.add(stdinIL, ilp);
    }
      // kill button
    RowLayoutProperties butp = new RowLayoutProperties(new Insets(14, 4, 6, 4), 0.06f);
    butp.leftSpaceGrowth = butp.rightSpaceGrowth = 0.43f;
    add(kill = new Button("Terminate !"));
    mainl.add(kill, butp);

    pack();

    stdoutTh.start();
    stderrTh.start();
    running = true;
    outputFinishedNotificator = stdoutTXA;
}

  public void show()
  {
    Dimension scr = Toolkit.getDefaultToolkit().getScreenSize(), me = size();
    move((scr.width - me.width) / 2, (int)((scr.height - me.height) / 2.4));
    super.show();
  }

  public boolean handleEvent(Event ev)
  {
    if(ev.id == Event.WINDOW_DESTROY && running)
    {
      new MessageBox("Warning", "Can not close now. The process must finish first.").show();
      return true;
    } else return super.handleEvent(ev);
  }

  public boolean action(Event ev, Object arg)
  {
    if(ev.target == kill)
    {
      sub.destroy();
      return true;
    }
    else if(ev.target == outputFinishedNotificator && ev.key == OutputTextArea.CLOSED)
    {
      running = false;
      setTitle(titleWhenFinished);
      sub = null;
      kill.enable(false);
      if(stdinIL != null)
      {
        stdinIL.enable(false);
        try { toClose.close(); } catch (IOException ex) {};
      }
      if(RunOptions.closeWhenFinished()) postEvent(new Event(this, Event.WINDOW_DESTROY, null));
      return true;
    }
    else return super.action(ev, arg);
  }

  public Insets insets()   // Under JDK 1.0.2 for win95 insets doesnt work. This patch should help.
  {
    Insets i = super.insets();
    if(getPeer() != null && (i.top < 0 || i.bottom < 0 || i.left < 0 || i.right < 0))
      return new Insets(28, 4, 4, 4);
    else return i;
  }
}














