/**
* Xelfi
*/

package xelfi.top.toolbar;

/**
  *
  * @version 	1.5, 25 Feb 1997
  * @author 	Petr Hamernik
  *
  */
import xelfi.top.MainMenu;

/** Maintain constants for toolbar e.g. filenames of icons
 *  and hints for buttons
 */
public class XFToolbarConstants {

	/** This value is added to event id for recognizing
	 *  events from toolbar
	 */
	public static final int TRANS = 60000;

	/** Gets the filename (without extension) of icon file
	 * @param index Index of item in menu.
	 * @return filename of the icon file.
	 */
	public static String getIconName(int index) {
		switch (index - TRANS) {
		    case MainMenu.miSystemNewClass:		return "new";
		    case MainMenu.miSystemSave:			return "save";
		    case MainMenu.miSystemSaveAll:		return "saveall";
		    case MainMenu.miSystemExit:			return "exit";
		    case MainMenu.miEditUndo:			return "undo";
		    case MainMenu.miEditCut:			return "cut";
		    case MainMenu.miEditCopy:			return "copy";
		    case MainMenu.miEditPaste:			return "paste";
			case MainMenu.miEditFind:			return "find";
			case MainMenu.miDesignShowGrid:		return "grid";
			case MainMenu.miDesignRealMode:		return "realmode";
		    case MainMenu.miCompile:			return "compile";
		    case MainMenu.miCompileAll:			return "compileall";
		    case MainMenu.miRunMain:			return "runmain";
		    case MainMenu.miDebugGo:			return "go";
		    case MainMenu.miDebugGoCursor:		return "gotocursor";
		    case MainMenu.miDebugFinish:		return "finish";
		    case MainMenu.miDebugRestart:		return "restart";
		    case MainMenu.miDebugTraceOver:		return "stepover";
		    case MainMenu.miDebugTraceInto:		return "traceinto";
		    case MainMenu.miDebugToggleBreak:	return "togglebreakpoint";
		    case MainMenu.miDebugAddWatch:		return "addwatch";
		    case MainMenu.miWindowWatches:		return "watches";
		    case MainMenu.miWindowLocals:		return "locals";
		    case MainMenu.miWindowCallStack:	return "callstack";
		    case MainMenu.miWindowThreads:		return "threads";
		    case MainMenu.miWindowBreakpoints:	return "breakpoints";
		    case MainMenu.miWindowOutput:		return "output";
		    case MainMenu.miWindowInspector:	return "ci";
		    case MainMenu.miSystemNewBrowser:	return "browser";
			default: return "question";
		}
	}

	/** Gets the hint of each command used in toolbar.
	 * @param index Index of item in menu.
	 * @return Hint.
	 */
	public static String getHint(int index) {
		switch (index - TRANS) {
		    case MainMenu.miSystemNewClass:		return "New from template";
		    case MainMenu.miSystemSave:			return "Save";
		    case MainMenu.miSystemSaveAll:		return "Save all";
		    case MainMenu.miSystemExit:			return "Exit Xelfi IDE";
		    case MainMenu.miEditUndo:			return "Undo";
		    case MainMenu.miEditCut:			return "Cut";
		    case MainMenu.miEditCopy:			return "Copy";
			case MainMenu.miEditPaste:			return "Paste";
			case MainMenu.miEditFind:			return "Find & Replace";
		    case MainMenu.miDesignShowGrid:		return "Show/Hide Grid";
		    case MainMenu.miDesignRealMode:		return "Switch between Real and Design Mode";
		    case MainMenu.miCompile:			return "Compile class";
		    case MainMenu.miCompileAll:			return "Compile All";
		    case MainMenu.miRunMain:			return "Run Main Class";
		    case MainMenu.miDebugGo:			return "Start debugging";
		    case MainMenu.miDebugGoCursor:		return "Go to cursor position";
		    case MainMenu.miDebugFinish:		return "Finish debugging session";
		    case MainMenu.miDebugRestart:		return "Restart debugging session";
		    case MainMenu.miDebugTraceOver:		return "Trace over";
		    case MainMenu.miDebugTraceInto:		return "Trace into";
		    case MainMenu.miDebugToggleBreak:	return "Toggle breakpoint";
		    case MainMenu.miDebugAddWatch:		return "Add watch";
		    case MainMenu.miWindowWatches:		return "Watches window";
		    case MainMenu.miWindowLocals:		return "Locals window";
		    case MainMenu.miWindowCallStack:	return "Callstack window";
		    case MainMenu.miWindowThreads:		return "Threads window";
		    case MainMenu.miWindowBreakpoints:	return "Breakpoints window";
		    case MainMenu.miWindowOutput:		return "Output window";
		    case MainMenu.miWindowInspector:	return "Component Inspector window";
		    case MainMenu.miSystemNewBrowser:	return "New browser window";
			default: return "who knows ???";
		}
	}
}
