/** Frame window for component palette
* @author: Filip Dvorak
* @version: 1.0
*/

package xelfi.top.visual;

import java.awt.*;

import xelfi.top.*;

import xelfi.design.components.palette.*;

public class PaletteWindow extends TopDialog
{
  protected XFComponentPalette palet;

  public PaletteWindow(Frame parent, XFComponentPalette pal)
  {
    super(parent, "Component palette", false);
    setLayout(new BorderLayout());
    add("Center", palet = pal);
  }

  public void show()
  {
    Rectangle par = getParent().bounds(), me = new Rectangle();
    Dimension pal = palet.preferredSize(), scr = Toolkit.getDefaultToolkit().getScreenSize();

    if(getPeer() == null) addNotify();  //because we need insets
    move(0, 0);
    resize(100, 100);  // to give insets chance to work

    me.width = insets().left + insets().right + pal.width;
    me.height = insets().top + insets().bottom + pal.height;
    me.x = ((par.width) - me.width)/2 + par.x;
    me.y = par.y + par.height;  //under the parent
    if(me.y + me.height > scr.height)
    {
      if(par.y - me.height > 0) me.y = par.y - me.height;
      else me.y -= me.y + me.height - scr.height;
    }
/*    System.out.println("Parent: " + par);
    System.out.println("Dock: " + me);
    System.out.println("pal pref size " + palet.preferredSize());
*/
    reshape(me.x, me.y, me.width + 5, me.height);  // Fix a bug
    super.show();
  }

  public boolean handleEvent(Event ev)
  {
    if(ev.target == this && ev.id == Event.WINDOW_DESTROY)
    {
      TopLevel.getMenu().itemAction(MainMenu.miDesignDock);  // disposes this
      return true;
    }
    else return super.handleEvent(ev);
  }

  public Insets insets()   // Under JDK 1.0.2 for win95 insets doesnt work. This patch should help.
  {
    Insets i = super.insets();
    if(getPeer() != null && (i.top < 0 || i.bottom < 0 || i.left < 0 || i.right < 0))
      return new Insets(28, 4, 4, 4);
    else return i;
  }
}






