/**
* Xelfi programming environment
*/

package xelfi.xedit;
import java.awt.*;
import xelfi.editor.*;
import xelfi.top.TopFrame;
/**
*
* @version 1.0
* @author Jaroslav Tulach
*/

public class EditorFrameBase extends TopFrame {
  /** the editor */
  TextEditor editor;

  /** the goto line dialog */
  private DialogGoToLine goTo;

  /** find dialog */
  private DialogFind find;

  /** Creates new editor window for specified document.
  * @param title the filename
  * @param doc the document if it is null a new one is loaded
  */
  protected EditorFrameBase (String title, Document doc) {
    super (title);

    add ("Center", editor = new TextEditor (doc, true));
    pack ();

    goTo = new DialogGoToLine (this, editor);
    find = new DialogFind (this, editor);
  };

  /** Sets focus to the editor
  */
  public void show () {
    super.show ();
    editor.requestFocus ();
  };

  /** Handle event.
  * @param ev the event
  */
  public boolean handleEvent (Event ev) {
    try {
      if (!super.handleEvent (ev)) {
        if (ev.id == Event.WINDOW_DESTROY) {
          dispose ();
          return true;
        };
        return false;
      };
    } catch (NullPointerException ex) {
    };
    return false;
  };

  /** redefine key handling method
  * @param e the event that happened
  * @param key the key pressed
  */
  public boolean keyDown (Event e, int key) {
    boolean ctrl = (e.modifiers & Event.CTRL_MASK) != 0;

    switch (key) {
    case 6: // Ctrl + F
      showFind ();
      return true;
    case 7: // Ctrl + G
      showGoTo ();
      return true;
    };
    return super.keyDown (e, key);
  };

  /** Returns the text editor in this frame.
  * @return the text editor
  */
  public TextEditor getTextEditor () {
    return editor;
  };

  /** Shows dialog for find.
  */
  public void showFind () {
    find.pack ();
    find.show ();
  };

  /** Shows go to line dialog.
  */
  public void showGoTo () {
    goTo.pack ();
    goTo.show ();
  };
};





